#ifndef INC_TEXTVIEW_H
#define INC_TEXTVIEW_H

#include "gtbdrman.h"
#include "gtdlink.h"
#include "gtcblist.h"

/************************************************************************\
*
* GtTextView class header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Structure of callback data */
typedef struct GtTextViewCD
{
	int txtxp,txtyp;
	int txtxs,txtys;
	int xsize,ysize;
	int topline,totline;
} GtTextViewCD;

/* Layout information for text view */
typedef struct GtWListTV
{
	GtWListBdr bwl;
	struct GtTVPart *tvp;
} GtWListTV;

/* Structure of a line header */
typedef struct GtTVLine
{
	GtDLink lnk;
	GtDLink phd;
	int xsize,ysize;
	int ypos;
	word yoff;
} GtTVLine;

/* Structure of a line part */
typedef struct GtTVPart
{
	GtDLink lnk;
	GtTVLine *tvl;
	int xpos;
	word xsize,ysize;
	ubyte type;
	union
	{
		struct
		{
			GtFont *fnt;
			ulong fg,bg;
			char *str;
			int len;
			word yoff;
			ubyte attr;
		} txt;
		struct
		{
			GtWListTV *wl;
			ubyte attr;
		} w;
	} d;
} GtTVPart;

/* Types of part */
#define GTTV_STRING		0
#define GTTV_WIDGET		1

/* Attributes for text */
#define GTTV_BOLD			0x01
#define GTTV_ULINE		0x02
#define GTTV_REVERSE	0x04
#define GTTV_FG				0x08
#define GTTV_BG				0x10
#define GTTV_MARK			0x40
#define GTTV_FREE			0x80

/* Maximum length of a string part */
#define GTTV_MAXSTRLEN	128


class GtTextView : public GtBdrMan
{
public:
		/* Standard */
	GtTextView(void);
	sbool Init(char *n,GtManager *p,GtTag *tgs);
	sbool Set(GtTag *tgs);
	void Get(GtTag *tgs);
	char *GetClass(void) { return("GtTextView"); }
	sbool IsSubClassOf(char *cn);
		/* Values */
	void SetEMask(ulong em);
	void GetTextSize(int *xs,int *ys) { *xs=lmxs; *ys=lmys; }
	void GetTextInc(int *xi,int *yi) { *xi=fw; *yi=fh; }
		/* Callbacks */
	void AddCB(ulong tag,GtCallB cb,GtUDPtr ud);
	void RemCB(ulong tag,GtCallB cb);
		/* Line handling */
	ulong AddLine(ulong pos);
	void RemLine(ulong l);
	void RemAllLines(void);
		/* Part handling */
	ulong AddPart(ulong l,char *str,int len,ulong pos);
	ulong AddPart(ulong l,GtWidget *w,ubyte attr,ulong pos);
	void RemPart(ulong p);
	void SetTextAttr(ulong p,GtFont *f,ubyte attr,ulong fg,ulong bg);
		/* Scrolling */
	void GetPosition(int *xp,int *yp) { *xp=left; *yp=top; }
	void SetPosition(int xp,int yp);
	void SetTopLine(ulong l);
	void SetTopLineNr(int nr);
		/* Child management */
	void RemChild(GtWidget *o);
		/* Child requests */
	void MoveSizeReq(GtWidget *o,word xp,word yp,word xs,word ys);
		/* Internal methods */
	void SetPosSize(word xp,word yp,word xs,word ys);
protected:
		/* Internal methods */
	sbool Draw(XEvent *xe);
	sbool Mouse(XEvent *xe);
	GtWList *MakeWList(GtWidget *o,int sz);
	void FindCursPos(int xp,int yp);
	void UpdateMark(sbool on);
	void RefreshRect(int xp,int yp,int xs,int ys);
private:
	void ReDraw(int xp,int yp,int xs,int ys);
	void MaxLineSizes(void);
	void MaxPartSizes(GtTVLine *tvl);
	void UpdateFont(void);
	void AllocMark(void);
	static int ConvMark(GtBase *obj,GtCDPtr cd,GtUDPtr ud);
	static int LoseMark(GtBase *obj,GtCDPtr cd,GtUDPtr ud);
	static int DoneMark(GtBase *obj,GtCDPtr cd,GtUDPtr ud);	
protected:
		/* Instance variables */
	GtCBList chg;
	GtDLink hd;
	GtTVLine *topl;
	GtTVPart *curtvp;
	GtTVPart *mrktvp;
	int curpos,curxp,curxs;
	int mrkpos,mrkxp,mrkxs;
	int top,left;
	int lmxs,lmys,topnr,totnr;
	int marklen;
	char *markstr;
	word fw,fh,foff;
	word tflags;
};

extern GtTextView *GtMkTextView(char *n,GtManager *p,GtTag *tgs);
extern GtTextView *GtMkTextView(char *n,GtManager *p,ulong tag,...);

#endif /* INC_GTTEXTVIEW_H */
