#include <proto.h>

#include "gtpulseknob.h"
#include "gtscroller.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtPulseKnob class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init pulseknob.
*
\************************************************************************/
GtPulseKnob *GtMkPulseKnob(char *n,GtManager *p,GtTag *tgs)
{
	GtPulseKnob *obj;
	
	if(!(obj=new GtPulseKnob()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtPulseKnob object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init pulseknob.
*
\************************************************************************/
GtPulseKnob *GtMkPulseKnob(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtPulseKnob *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkPulseKnob(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty pulseknob.
*
\************************************************************************/
GtPulseKnob::GtPulseKnob(void)
{
	del=50;
}

/************************************************************************\
*
* Destroy pulseknob.
*
\************************************************************************/
void GtPulseKnob::Destroy(void)
{
	if(down)
		app->RemTimeOut(GtPulseKnob::Pulse);
	GtKnob::Destroy();
}

/************************************************************************\
*
* Setup pulseknob.
*
\************************************************************************/
sbool GtPulseKnob::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtKnob::Init(n,p,tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Delay:
				if(t->d.val>0)
					del=t->d.val;
				break;
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Set pulseknob attributes.
*
\************************************************************************/
sbool GtPulseKnob::Set(GtTag *tgs)
{
	GtTag *t;
	
	if(!GtKnob::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Delay:
				if(t->d.val>0)
					del=t->d.val;
				break;
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Get pulseknob attributes.
*
\************************************************************************/
void GtPulseKnob::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtKnob::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Delay:
				*ptr=del;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtPulseKnob::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtPulseKnob")==0)
		return(TRUE);
	else
		return(GtKnob::IsSubClassOf(cn));
}

/************************************************************************\
*
* Notify from timeout to activate scroller.
*
\************************************************************************/
void GtPulseKnob::Notify(ulong tag)
{
	if(tag==GT_Activate)
	{
		parent->Notify(GT_Activate);
		app->AddTimeOut(del,GtPulseKnob::Pulse,this);
	}
	else
		GtKnob::Notify(tag);
}

/************************************************************************\
*
* Draw pulseknob.
*
\************************************************************************/
sbool GtPulseKnob::Draw(XEvent *xe)
{
	int xp,yp,sz;
	
	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	GtKnob::Draw(xe);
	sz=GTMIN(xsize-2*bdrw,ysize-2*bdrw);
	xp=bdrw+(xsize-2*bdrw-sz)/2;
	yp=bdrw+(ysize-2*bdrw-sz)/2;
	gtlht->DrawArc(gtwin,xp,yp,sz-1,sz-1,45*64,180*64,FALSE);
	gtdrk->DrawArc(gtwin,xp,yp,sz-1,sz-1,(180+45)*64,180*64,FALSE);
	return(TRUE);
}

/************************************************************************\
*
* Mouse pulseknob events.
*
\************************************************************************/
sbool GtPulseKnob::Mouse(XEvent *xe)
{
	switch(xe->type)
	{
		case ButtonPress:
			if(xe->xbutton.button==Button1)
			{
				down=TRUE;
				xoff=xe->xbutton.x;
				yoff=xe->xbutton.y;
				dsp->SetFocus(this);
				this->Refresh();
				app->AddTimeOut(del,GtPulseKnob::Pulse,this);
				return(TRUE);
			}
			break;
		case ButtonRelease:
			if(xe->xbutton.button==Button1)
			{
				down=FALSE;
				dsp->ClrFocus(this);
				this->Refresh();
				parent->SetV(GT_Value,0,GT_DONE);
				parent->Notify(GT_Activate);
				app->RemTimeOut(GtPulseKnob::Pulse);
				return(TRUE);
			}
			break;
		case MotionNotify:
			if(down)
			{
				this->Move(xpos+(xe->xmotion.x-xoff),ypos+(xe->xmotion.y-yoff));
				return(TRUE);
			}
			break;
	}
	return(FALSE);
}

/************************************************************************\
*
* Send pulse to scroller.
*
\************************************************************************/
int GtPulseKnob::Pulse(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtPulseKnob *pn;
	
	obj=obj;
	cd=cd;
	pn=(GtPulseKnob *)ud;
	pn->parent->Notify(GT_Activate);
	pn->app->AddTimeOut(pn->del,GtPulseKnob::Pulse,pn);
/*
	((GtPulseKnob *)ud)->Notify(GT_Activate);
*/
	return(GTCB_CONT);
}

